@@ -2,6 +2,8 @@ import os
 from typing import List
 
 import requests
+from requests.adapters import HTTPAdapter
+from urllib3 import Retry
 
 from maitai._config_listener_thread import ConfigListenerThread
 from maitai._utils import __version__ as version
@@ -16,6 +18,16 @@ class Config:
     maitai_ws = os.environ.get("MAITAI_WS", 'wss://09hidyy627.execute-api.us-west-2.amazonaws.com/production')
 
     def __init__(self):
+        retry_strategy = Retry(
+            total=3,
+            backoff_factor=1
+        )
+        adapter = HTTPAdapter(max_retries=retry_strategy)
+
+        self._session = requests.Session()
+        self._session.mount("http://", adapter)
+        self._session.mount("https://", adapter)
+
         self._api_key = None
         self._applications: dict[str, Application] = {}
         self._company_id = None
@@ -46,11 +58,13 @@ class Config:
         self._api_key = value
 
     def initialize(self, api_key):
+        if self.initialized and self.api_key == api_key:
+            return
         self.api_key = api_key
+        self.initialized = True
         self._initialize_company()
         self.refresh_applications()
         self._initialize_websocket()
-        self.initialized = True
 
     def get_application(self, application_ref_name: str) -> Application:
         return self._applications.get(application_ref_name)
@@ -67,7 +81,8 @@ class Config:
             'x-client-version': version,
         }
 
-        response = requests.get(url, headers=headers, timeout=10)
+        response = self._session.get(url, headers=headers, timeout=15)
+        self._session.close()
         if response.status_code != 200:
             print(f"Failed to initialize API client. Status code: {response.status_code}. Error: {response.text}")
             return
@@ -83,7 +98,8 @@ class Config:
             'x-client-version': version,
         }
 
-        response = requests.get(url, headers=headers, timeout=10)
+        response = self._session.get(url, headers=headers, timeout=15)
+        self._session.close()
         if response.status_code != 200:
             print(f"Failed to initialize API client. Status code: {response.status_code}. Error: {response.text}")
             return
