@@ -3,6 +3,7 @@ from typing import AsyncIterable, Iterable, Optional
 
 import aiohttp
 import requests
+from aiohttp import ClientTimeout
 from betterproto import Casing
 
 from maitai._config import config
@@ -38,10 +39,10 @@ class Inference(MaitaiClient):
     def infer(cls, chat_request: ChatCompletionRequest, evaluate_callback, timeout) -> Iterable[InferenceStreamResponse]:
         if evaluate_callback:
             q = ChunkQueue()
-            cls.run_async(cls.send_inference_request_async(chat_request, chunk_queue=q, evaluation_callback=evaluate_callback))
+            cls.run_async(cls.send_inference_request_async(chat_request, chunk_queue=q, timeout=timeout, evaluation_callback=evaluate_callback))
             return QueueIterable(q, timeout=timeout)
         else:
-            return cls.send_inference_request(chat_request)
+            return cls.send_inference_request(chat_request, timeout)
 
     @classmethod
     async def infer_async(cls, chat_request: ChatCompletionRequest, evaluate_callback, timeout) -> AsyncIterable[InferenceStreamResponse]:
@@ -65,7 +66,7 @@ class Inference(MaitaiClient):
         cls.run_async(cls.send_storage_request_async(chat_storage_request))
 
     @classmethod
-    def send_inference_request(cls, chat_request: ChatCompletionRequest) -> Iterable[InferenceStreamResponse]:
+    def send_inference_request(cls, chat_request: ChatCompletionRequest, timeout) -> Iterable[InferenceStreamResponse]:
         def consume_stream():
             host = config.maitai_host
             url = f'{host}/chat/completions/serialized'
@@ -74,7 +75,7 @@ class Inference(MaitaiClient):
                 'x-api-key': config.api_key,
                 'x-client-version': version,
             }
-            response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), stream=True)
+            response = requests.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE), stream=True, timeout=timeout or 120)
             if response.status_code != 200:
                 raise MaitaiConnectionError(f"Failed to send inference request. Status code: {response.status_code}. Error: {response.text}")
             try:
@@ -86,10 +87,12 @@ class Inference(MaitaiClient):
 
         for resp in consume_stream():
             inference_response: InferenceStreamResponse = InferenceStreamResponse().from_json(resp)
+            if inference_response.keep_alive:
+                continue
             yield inference_response
 
     @classmethod
-    async def send_inference_request_async(cls, chat_request: ChatCompletionRequest, chunk_queue: ChunkQueue = None, async_chunk_queue: AsyncChunkQueue = None, evaluation_callback: EvaluateCallback = None):
+    async def send_inference_request_async(cls, chat_request: ChatCompletionRequest, chunk_queue: ChunkQueue = None, async_chunk_queue: AsyncChunkQueue = None, evaluation_callback: EvaluateCallback = None, timeout=None):
         host = config.maitai_host
         url = f'{host}/chat/completions/serialized'
         headers = {
@@ -97,7 +100,7 @@ class Inference(MaitaiClient):
             'x-api-key': config.api_key,
             'x-client-version': version,
         }
-        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False)) as session:
+        async with aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False), timeout=ClientTimeout(timeout or 120)) as session:
             async with session.post(url, headers=headers, data=chat_request.to_json(casing=Casing.SNAKE)) as response:
                 if response.status != 200:
                     error_text = await response.text()
@@ -110,6 +113,8 @@ class Inference(MaitaiClient):
                 async for line in response.content:
                     if line:
                         inference_response: InferenceStreamResponse = InferenceStreamResponse().from_json(line)
+                        if inference_response.keep_alive:
+                            continue
                         if chunk_queue:
                             chunk_queue.put(inference_response)
                         if async_chunk_queue:
